package main;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Graphics;
import java.awt.HeadlessException;
import java.awt.Rectangle;
import java.awt.Robot;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import java.nio.charset.StandardCharsets;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.util.HashMap;
import java.util.Map;

import javax.imageio.ImageIO;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.SwingUtilities;

public class Main {

	private static BufferedImage checkBoxImage;
	private static Robot robo;

	private static BuscadorDaCheckboxDaLinhaSelecionada buscadorDaCheckboxDaLinhaSelecionada;
	private static Clicador clicador;

	private static Color corDeFundo = Color.DARK_GRAY;
	
	private static int larguraDaTela;
	private static int alturaDaTela;
	private static int posXdaTela;
	private static int posYdaTela;

	public static void main(String[] args) {

		try {
			iniciarConfiguracoes();
			clicador = new Clicador(robo, buscadorDaCheckboxDaLinhaSelecionada);
		} catch (Exception e1) {
			e1.printStackTrace();
			saidaComErro(e1);
		}

		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				try {
					new Tela();
				} catch (Exception e) {
					e.printStackTrace();
					saidaComErro(e);
				}
			}
		});
	}

	/** Altere as configuraes para corresponder as suas necessidades. */
	private static void iniciarConfiguracoes() throws Exception {
		robo = new Robot();
		carregarConfiguracoesDoArquivo();
	}

	private static void carregarConfiguracoesDoArquivo() throws IOException {
		carregarConfiguracaoesAPartirDoMap(carregarMapDeConfiguracoes());
	}

	private static void carregarConfiguracaoesAPartirDoMap(Map<String, String> configuracoes) throws IOException {
		Conversor c = new Conversor(configuracoes);

		checkBoxImage = ImageIO.read(new File(configuracoes.get("IMAGEM_DE_UMA_CHECKBOX")));

		BuscadorDeLinhaSelecionada buscadorDeLinhaSelecionada = new BuscadorDeLinhaSelecionada(
				new Color(
						c.integerConfig("COR_DA_SELECAO_RED"), 
						c.integerConfig("COR_DA_SELECAO_GREEN"), 
						c.integerConfig("COR_DA_SELECAO_BLUE")), 
				c.integerConfig("MARGEM_SUPERIOR_ADICIONADA_A_AREA_DA_CHECKBOX_BUSCADA"), 
				c.integerConfig("MARGEM_INFERIOR_ADICIONADA_A_AREA_DA_CHECKBOX_BUSCADA"));

		Rectangle areaDaSelecao = new Rectangle(
				c.integerConfig("AREA_DA_SELECAO_X"), 
				c.integerConfig("AREA_DA_SELECAO_Y"), 
				c.integerConfig("AREA_DA_SELECAO_WIDTH"), 
				c.integerConfig("AREA_DA_SELECAO_HEIGHT"));

		buscadorDaCheckboxDaLinhaSelecionada = new BuscadorDaCheckboxDaLinhaSelecionada(
				checkBoxImage, buscadorDeLinhaSelecionada, areaDaSelecao, 
				c.integerConfig("BORDA_ESQUERDA_DA_AREA_DE_CHECKBOX"), 
				c.integerConfig("BORDA_DIREITA_DA_AREA_DE_CHECKBOX"));

		larguraDaTela = c.integerConfig("LARGURA_DA_JANELA");
		alturaDaTela = c.integerConfig("ALTURA_DA_JANELA");
		posXdaTela = c.integerConfig("POSICAO_X_DA_JANELA");
		posYdaTela = c.integerConfig("POSICAO_Y_DA_JANELA");
	}

	private static class Conversor {
		Map<String, String> configuracoes;

		Conversor(Map<String, String> configuracoes) {
			this.configuracoes = configuracoes;
		}

		int integerConfig(String key) {
			return Integer.parseInt(configuracoes.get(key));
		}
	}

	private static Map<String, String> carregarMapDeConfiguracoes() throws IOException {
		Map<String, String> configuracoes = new HashMap<>();
		Files.lines(Paths.get("src/configuracoes.txt"), StandardCharsets.ISO_8859_1).forEach(linha -> {
			if (!linha.isEmpty() && !linha.startsWith("//")) {
				System.out.println(linha);
				String[] split = linha.trim().split(":");
				String key = split[0].trim();
				String value = split[1].trim();
				if (value.startsWith("\"") || value.startsWith("\'") || value.startsWith("")) {
					value = value.substring(1);
				}
				if (value.endsWith("\"") || value.endsWith("\'") || value.endsWith("")) {
					value = value.substring(0, value.length() - 1);
				}
				System.out.println("'" + key + "':'" + value + "'");
				configuracoes.put(key, value);
			}
		});
		return configuracoes;
	}

	private static void saidaComErro(Exception e) throws HeadlessException {
		JOptionPane.showMessageDialog(null, e);
		System.exit(1);
	}

	@SuppressWarnings("serial")
	private static class Tela extends JFrame {

		Tela() {
			super("Clicador em Checkbox de linha Selecionada");
			System.out.println("creating instance");
			setSize(larguraDaTela, alturaDaTela);
			setLocation(posXdaTela, posYdaTela);
			setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
			add(new Painel(), BorderLayout.CENTER);
			JButton jButton = new JButton("Clicar na Checkbox!");
			jButton.addActionListener(e -> {
				try {
					boolean clicou = clicador.clicarNaCheckbox();
					corDeFundo = (clicou) ? new Color(0, 127, 0) : new Color(127, 0, 0);
					repaint();
					restaurarCorDeFundoNeutraApos(1000);
				} catch (Exception ex) {
					saidaComErro(ex);
				}
			});
			add(jButton, BorderLayout.SOUTH);
			setAlwaysOnTop(true);
			setVisible(true);
		}

		private void restaurarCorDeFundoNeutraApos(long millis) {
			new Thread(() -> {
				try {
					Thread.sleep(millis);
				} catch (InterruptedException e1) {
					e1.printStackTrace();
				}
				corDeFundo = Color.DARK_GRAY;
				SwingUtilities.invokeLater(() -> repaint());
			}).start();
		}
	}

	@SuppressWarnings("serial")
	private static class Painel extends JPanel {
		@Override
		protected void paintComponent(Graphics g) {
			super.paintComponent(g);
			g.setColor(corDeFundo);
			g.fillRect(0, 0, getWidth(), getHeight());

			int posYincrement = 0;
			BufferedImage selecaoEncontrada = buscadorDaCheckboxDaLinhaSelecionada.selecaoEncontrada.get();
			if (selecaoEncontrada != null) {
				g.drawImage(selecaoEncontrada, 20, 20, null);
				posYincrement = selecaoEncontrada.getHeight() + 10;
			}
			
			BufferedImage areaDaSelecao = buscadorDaCheckboxDaLinhaSelecionada.areaDaSelecaoBuscada.get();
			if (areaDaSelecao != null) {
				g.drawImage(areaDaSelecao, 20, posYincrement+20, null);
				BufferedImage areaDaCheckbox = buscadorDaCheckboxDaLinhaSelecionada.areaComCheckboxBuscada.get();
				if (areaDaCheckbox != null) {
					g.drawImage(areaDaCheckbox, areaDaSelecao.getWidth() + 30, 20, null);
					g.drawImage(checkBoxImage, areaDaSelecao.getWidth() + 30, areaDaCheckbox.getHeight() + 30, null);
				}
			}
		}

	}

}
